import numpy as np
from typing import List, Optional
from .math_utils import translation, rotation_x, rotation_y, rotation_z


def get_dome_cameras(
    rows: int,
    columns: int,
    radius: float,
    transform_matrix: Optional[np.ndarray] = None,
    output_format: str = "nerfstudio",
) -> List[np.ndarray]:
    cameras: List[np.ndarray] = []

    if transform_matrix is None:
        transform_matrix = np.eye(4)

    for row in range(rows):
        for col in range(columns):
            # default camera at (0, 0, 0) looking towards -z
            w2c = np.eye(4)

            if output_format == "nerfstudio":
                # move the camera back by radius
                w2c = translation(0, -radius, 0) @ w2c
                # rotate around x axis (up and down) from top to middle
                w2c = rotation_x(-np.pi / 2 * (row / (rows - 1))) @ w2c
                # rotate around z axis (left and right)
                w2c = rotation_z(2 * np.pi * col / columns) @ w2c
                # move the "sphere" where we want it
                w2c = transform_matrix @ w2c
            else:
                # move the camera back by radius
                w2c = w2c @ translation(0, 0, radius)
                # rotate around x axis (up and down)
                w2c = w2c @ rotation_x(np.pi * row / (2 * (rows - 1)))
                # rotate around y axis (left and right)
                w2c = w2c @ rotation_y(2 * np.pi * col / columns)
                # move the "sphere" where we want it
                w2c = w2c @ transform_matrix

            cameras.append(w2c)

    return cameras


def get_sphere_cameras(
    rows: int,
    columns: int,
    radius: float,
    transform_matrix: Optional[np.ndarray] = None,
    output_format: str = "nerfstudio",
) -> List[np.ndarray]:
    cameras: List[np.ndarray] = []

    if transform_matrix is None:
        transform_matrix = np.eye(4)

    for row in range(rows):
        for col in range(columns):
            # default camera at (0, 0, 0) looking towards -z
            w2c = np.eye(4)

            if output_format == "nerfstudio":
                # make the camera look up towards +y
                w2c = rotation_x(np.pi / 2) @ w2c
                # move the camera back by radius
                w2c = translation(0, -radius, 0) @ w2c
                # rotate around x axis
                w2c = rotation_x(np.pi / 2 - np.pi * row / (rows - 1)) @ w2c
                # # rotate around z axis
                w2c = rotation_z(2 * np.pi * col / columns) @ w2c
                # # move the "sphere" where we want it
                w2c = transform_matrix @ w2c
            else:
                # move the camera back by radius
                w2c = w2c @ translation(0, 0, radius)
                # rotate around x axis
                w2c = w2c @ rotation_x(-np.pi / 2 + np.pi * row / (rows - 1))
                # rotate around y axis
                w2c = w2c @ rotation_y(2 * np.pi * col / columns)
                # move the "sphere" where we want it
                w2c = w2c @ transform_matrix

            cameras.append(w2c)

    return cameras


def get_cameras(
    rows: int,
    columns: int,
    radius: float,
    structure_type: str,
    output_format: str = "nerfstudio",
    transform_matrix: Optional[np.ndarray] = None,
) -> List[np.ndarray]:
    if transform_matrix is None:
        transform_matrix = np.eye(4)

    if structure_type == "dome":
        return get_dome_cameras(rows, columns, radius, transform_matrix, output_format)
    elif structure_type == "sphere":
        return get_sphere_cameras(
            rows, columns, radius, transform_matrix, output_format
        )
    else:
        raise ValueError("Invalid structure type")
