import bpy
import numpy as np
from bpy.app.handlers import persistent
import time

from all_axis_render_camera.get_cameras import get_cameras


def setup_camera_positions(
    context, rows, cols, radius, structure_type, center_type, center_object
):
    context.scene.frame_end = rows * cols
    if "all_axis_cam" not in bpy.data.objects:
        new_cam = bpy.data.cameras.new(name="all_axis_cam")
        new_cam_object = bpy.data.objects.new("all_axis_cam", new_cam)
        context.scene.collection.objects.link(new_cam_object)

    cam_object = bpy.data.objects["all_axis_cam"]
    # cam_object.location.z = 10  # Move the camera up by 10 units
    # cam_object.rotation_euler = (0, 0, 0)  # Set rotation to look forward

    transform_matrix = np.eye(4)

    if center_type == "cursor":
        transform_matrix[:3, 3] = context.scene.cursor.location
    elif center_type == "object" and center_object:
        transform_matrix[:3, 3] = center_object.location
        rotation_matrix = center_object.matrix_world.to_3x3().normalized()
        transform_matrix[:3, :3] = rotation_matrix
    elif center_type == "origin":
        pass
    else:
        raise ValueError("Invalid center type")

    camera_locations = get_cameras(
        rows, cols, radius, structure_type, transform_matrix=transform_matrix
    )

    # Reset all camera keyframes
    cam_object.animation_data_clear()

    for frame_number, camera_matrix in enumerate(camera_locations, start=1):
        # camera_matrix[:3, 3] += center  # Adjust the camera position by the center
        cam_object.matrix_world = camera_matrix.T  # Blender is using column major order
        cam_object.keyframe_insert(data_path="location", frame=frame_number)
        cam_object.keyframe_insert(data_path="rotation_euler", frame=frame_number)

    if (
        context.scene.frame_current < 1
        or context.scene.frame_current > context.scene.frame_end
    ):
        context.scene.frame_set(1)
    else:
        context.scene.frame_set(context.scene.frame_current)


last_update_time = 0
update_delay = 0.1  # Delay in seconds


def update_camera_setup(scene):
    global last_update_time

    if "all_axis_cam" not in bpy.data.objects:
        return

    cam_setup_context = scene.cam_setup
    last_update_time = time.time()

    def delayed_setup():
        if time.time() - last_update_time >= update_delay:
            setup_camera_positions(
                bpy.context,
                cam_setup_context.cam_rows,
                cam_setup_context.cam_columns,
                cam_setup_context.radius,
                cam_setup_context.structure_type,
                cam_setup_context.center_type,
                cam_setup_context.center_object,
            )

    bpy.app.timers.register(delayed_setup, first_interval=update_delay)


@persistent
def update_camera_setup_depsgraph_handler(scene, depsgraph):
    cam_setup_context = scene.cam_setup

    if cam_setup_context.center_type != "object":
        return

    if not cam_setup_context.camera_setup_follows_object:
        return

    cam_setup_context = scene.cam_setup
    target_obj = cam_setup_context.center_object
    if target_obj is None:
        return

    if target_obj.name in [update.id.name for update in depsgraph.updates]:
        update_camera_setup(scene)
